

#include <bedslope.h>

#include <osg/BoundingBox>
#include <osg/Texture2D>
#include <osgDB/Registry>
#include <osgDB/ReadFile>
#include <osg/TexEnv>

#define DEF_BEDSLOPE_COLOUR     145, 122, 59, 255     // R, G, B, Alpha (brown)


// constructor
BedSlope::BedSlope(SWWReader* sww)
{

    // bedslope geometry as triangles with shared vertices
    _geom = new osg::Geometry;

    // geometry from sww file
    osg::Vec3Array* vertices = sww->getBedslopeVertexArray().get();
    osg::UIntArray* indices = sww->getBedslopeIndexArray().get();
    osg::Vec3Array* normals = sww->getBedslopeNormalArray().get();
    unsigned int nindices = sww->getNumberOfBedslopeIndices();

    _geom->setUseDisplayList(false);
    _geom->setVertexArray( vertices );
    _geom->setVertexIndices( indices );
    _geom->addPrimitiveSet(new osg::DrawArrays(osg::PrimitiveSet::TRIANGLES, 0, nindices ));
    _geom->setNormalArray( normals );
    _geom->setNormalBinding( osg::Geometry::BIND_PER_PRIMITIVE );


    // geometry bounding box
    osg::BoundingBox bbox = _geom->getBound();

    // bedslope texture coords scaled to range [0,1]
    osg::Vec2Array* texcoords = new osg::Vec2Array;
    osg::Vec3 v;
    float xrange = bbox.xMax() - bbox.xMin();
    float yrange = bbox.yMax() - bbox.yMin();
    for( unsigned int i=0; i < vertices->size(); i++ )
    {
        v = vertices->at(i);
        texcoords->push_back( osg::Vec2( (v.x()-bbox.xMin())/xrange, (v.y()-bbox.yMin())/yrange) );
    }
    _geom->setTexCoordArray( 0, texcoords );
    _geom->setTexCoordIndices( 0, indices );

    // bedslope texture
    osg::Texture2D* texture = new osg::Texture2D;
    texture->setDataVariance(osg::Object::DYNAMIC);
    texture->setWrap(osg::Texture::WRAP_S, osg::Texture::REPEAT);
    texture->setWrap(osg::Texture::WRAP_T, osg::Texture::REPEAT);
    texture->setImage(osgDB::readImageFile("bedslope.jpg"));


    // bedslope colour
    //osg::UByte4Array* colors = new osg::UByte4Array;  
    //colors->push_back(osg::UByte4(DEF_BEDSLOPE_COLOUR));
    //_geom->setColorArray(colors);
    //_geom->setColorBinding(osg::Geometry::BIND_OVERALL);

    // material properties
    _material = new osg::Material;
    _material->setDiffuse(osg::Material::FRONT, osg::Vec4(DEF_BEDSLOPE_COLOUR));
    _material->setSpecular(osg::Material::FRONT, osg::Vec4(1,0,0,1));
    _material->setShininess(osg::Material::FRONT, 128);

    
    _stateset = new osg::StateSet;
    _stateset->setTextureAttributeAndModes( 0, texture, osg::StateAttribute::ON );
    _stateset->setMode( GL_BLEND, osg::StateAttribute::ON );
    _stateset->setMode( GL_LIGHTING, osg::StateAttribute::ON );
    _stateset->setAttribute(_material);




    _node = new osg::Geode;
    _node->setName("bedslope");
    _node->addDrawable(_geom);
    _node->setStateSet(_stateset);
}

