
/*
    HeadsUpDisplay class for orthographic foreground display of text

    An OpenSceneGraph viewer for pyVolution SWW files.
    copyright (C) 2004 Geoscience Australia
*/


#include <osg/Geode>
#include <osg/Geometry>
#include <osg/Group>
#include <osg/MatrixTransform>
#include <osg/StateSet>
#include <hud.h>

#define ORTHO2D_WIDTH 1280
#define ORTHO2D_HEIGHT 1024
#define DEF_HUD_COLOUR   0.8, 0.8, 0.8, 1.0


// constructor
HeadsUpDisplay::HeadsUpDisplay()
{
    // a heads-up display requires an orthographic projection
    _projection = new osg::Projection;
    _projection->setMatrix( osg::Matrix::ortho2D(0,ORTHO2D_WIDTH,0,ORTHO2D_HEIGHT) );

    // font
    osgText::Font* font = osgText::readFontFile("fonts/arial.ttf");

    // title text
	_title = new osgText::Text;
    _title->setFont(font);
    _title->setColor(osg::Vec4(DEF_HUD_COLOUR) );
    _title->setCharacterSize(20);
    _title->setPosition(osg::Vec3(20,20,0));
    _title->setFontResolution(40,40);

    // timer text
    _time = new osgText::Text;
    _time->setFont(font);
    _time->setColor(osg::Vec4(DEF_HUD_COLOUR) );
    _time->setCharacterSize(30);
    _time->setPosition(osg::Vec3(1100,20,0));
    _time->setFontResolution(40,40);
    _time->setText("t = 0.0");

    // state
    osg::StateSet *state = _projection->getOrCreateStateSet();
    state->setRenderBinDetails(12, "RenderBin");
    state->setMode(GL_DEPTH_TEST,osg::StateAttribute::OFF);
    state->setMode( GL_LIGHTING, osg::StateAttribute::OFF );

    // scenegraph
    osg::MatrixTransform* xfm = new osg::MatrixTransform;
    xfm->setReferenceFrame(osg::Transform::RELATIVE_TO_ABSOLUTE);
    xfm->setMatrix(osg::Matrix::identity());
    osg::Geode* textnode = new osg::Geode;
    textnode->addDrawable( _title );
    textnode->addDrawable( _time );
    xfm->addChild( textnode );
    _projection->addChild( xfm );
}


HeadsUpDisplay::~HeadsUpDisplay()
{
}



void HeadsUpDisplay::setTime( float t )
{
	char timestr[20];
	sprintf(timestr, "t = %-7.2f", t);
	_time->setText(timestr);
}


void HeadsUpDisplay::setTitle( char *s )
{
    _title->setText(s);
}
